/* public domain code */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define USAGE \
	"usage: gbdelogo [<option(s)>] <romfile>\n" \
	"options:\n" \
	"    -?,   --help    Display this message.\n" \
	"    -p=#, --pad=#   Overwrite Nintendo logo with padding.\n" \
	"    -l,   --logo    Restore valid Nintendo logo data.\n" \
	"Default action is to overwrite Nintendo logo with 0xFF.\n"

static const unsigned char LOGO[] = {
	0xCE, 0xED, 0x66, 0x66, 0xCC, 0x0D, 0x00, 0x0B,
	0x03, 0x73, 0x00, 0x83, 0x00, 0x0C, 0x00, 0x0D,
	0x00, 0x08, 0x11, 0x1F, 0x88, 0x89, 0x00, 0x0E,
	0xDC, 0xCC, 0x6E, 0xE6, 0xDD, 0xDD, 0xD9, 0x99,
	0xBB, 0xBB, 0x67, 0x63, 0x6E, 0x0E, 0xEC, 0xCC,
	0xDD, 0xDC, 0x99, 0x9F, 0xBB, 0xB9, 0x33, 0x3E
};

int main(int argc, char **argv) {
	char *filename = NULL;
	unsigned char pad = -1;
	int dologo = 0;
	unsigned char data[0x30];
	FILE *file = NULL;
	int i;
	int len;
	char *s;
	for (i = 1; i < argc; i++) {
		s = argv[i];
		if (strcmp(s, "-?") == 0 || strcmp(s, "--help") == 0) {
			fprintf(stderr, USAGE);
			return EXIT_SUCCESS;
		} else if (strcmp(s, "-l") == 0 || strcmp(s, "--logo") == 0) {
			dologo = 1;
		} else if (
			strncmp(s, "-p=", (len = 3)) == 0 ||
			strncmp(s, "--pad=", (len = 6)) == 0
		) {
			s += len;
			pad = strtol(s, NULL, 0);
			dologo = 0;
		} else {
			filename = s;
		}
	}
	if (!filename) {
		fprintf(stderr, "No file specified.\n\n");
		fprintf(stderr, USAGE);
		return EXIT_FAILURE;
	}
	file = fopen(filename, "rb+");
	if (!file) {
		fprintf(stderr, "Can't read/write \"%s\".\n\n", filename);
		fprintf(stderr, USAGE);
		return EXIT_FAILURE;
	}
	if (fseek(file, 0x104, SEEK_SET)) {
		fclose(file);
		fprintf(stderr, "Can't write Nintendo logo header.\n\n");
		fprintf(stderr, USAGE);
		return EXIT_FAILURE;
	}
	dologo ? memcpy(data, LOGO, 48) : memset(data, pad, 48);
	fwrite(data, 1, 48, file);
	fclose(file);
	if (dologo)
		fprintf(stderr, "Nintendo logo data restoration successful.\n");
	else
		fprintf(stderr, "Overwrote Nintendo logo data with 0x%02X.\n",
			pad);
	return EXIT_SUCCESS;
}


